<?php
namespace Teknisa\Model\Services\Repositories;


class FctContrato extends \Doctrine\ORM\EntityRepository {


    public function findContractItemInfo($nrApropAgendamento) {
        $query = <<<SQL
            SELECT CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS CONTRACT_DESC,
                   IC.NMITEMCONTRATO, (SELECT HORADEC_HORACHAR(IC.QTMEDIDACONTR) FROM DUAL) AS QTMEDIDACONTR,
                   round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED,
                   round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED_ORIGIN,
                   AG.NRAPROPAGENDAMENTO, TO_CHAR(AG.DTAPROPRIACAO, 'DD/MM/YYYY') AS DTAPROPRIACAO,
                   (SELECT HORADEC_HORACHAR(AG.QTHORASAPROPRIADAS) FROM DUAL) AS QTHORASAPROPRIADAS,
                   (SELECT HORADEC_HORACHAR(AA.TOTAL_APPROPRIATION) FROM DUAL) AS TOTAL_APPROPRIATION,
                   RA.NRRECURSOSAGENDCONT, RA.NRPARCNEPESSOA
              FROM FCT_APROPAGENDAMENTO  AG
              JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRRECURSOSAGENDCONT = AG.NRRECURSOSAGENDCONT
              JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
              JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
              JOIN (SELECT SUM(AA.QTHORASAPROPRIADAS) AS TOTAL_APPROPRIATION, AA.NRRECURSOSAGENDCONT
                      FROM FCT_APROPAGENDAMENTO AA
                  GROUP BY AA.NRRECURSOSAGENDCONT) AA ON (AA.NRRECURSOSAGENDCONT = AG.NRRECURSOSAGENDCONT)
             WHERE AG.NRAPROPAGENDAMENTO = :NRAPROPAGENDAMENTO
SQL;

        $params = array(
            'NRAPROPAGENDAMENTO' => $nrApropAgendamento
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

    }

    public function findContractByResource($userId, $filterDeafult, $filter, $lessThanHundred = null)
    {
        $lessThanHundredFilter = '';
        if($lessThanHundred) {
            $lessThanHundredFilter = 'AND DECODE(IC.QTMEDIDACONTR, 0, 100,(TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR) < 100';
        }

        $query = <<<SQL
            SELECT *
            FROM (SELECT DISTINCT CO.NRSEQCONTRATO,
                                  CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS CONTRACTO_DESC
                             FROM RESOURCE_T            RE
                             JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
                             JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                             JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
                             JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                             JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
                             JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
                             JOIN (SELECT SUM(AA.QTHORASAPROPRIADAS) AS TOTAL_APPROPRIATION, AA.NRRECURSOSAGENDCONT
                                     FROM FCT_APROPAGENDAMENTO AA
                                 GROUP BY AA.NRRECURSOSAGENDCONT) AA ON (AA.NRRECURSOSAGENDCONT = AG.NRRECURSOSAGENDCONT)
                            WHERE RE.ID = :USER_ID
                              AND RA.IDSITUARECGEND = '1'
                             $filterDeafult
                             $lessThanHundredFilter)
            WHERE 1 = 1
            $filter
SQL;

        $params = array(
            'USER_ID' => $userId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findContractsWithAppropriateServiceAtDay($userId, $currentDate, $filter, $appropriationTime) {
        $query = <<<SQL
            SELECT ID, NAME, SUM(TOTAL_FAT) AS BILLED, SUM(TOTAL_NOT_FAT) AS NOT_BILLED, (SUM(TOTAL_FAT) + SUM(TOTAL_NOT_FAT)) AS TOTAL
                     FROM (SELECT CO.NRSEQCONTRATO AS ID, 'CONTRATO - ' || CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS NAME,
                                  (AG.QTHORASAPROPRIADAS * 3600) AS TOTAL_FAT, 0 AS TOTAL_NOT_FAT, 'A' AS DIFF
                             FROM RESOURCE_T            RE
                             JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
                             JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                             JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
                             JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                             JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
                             JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
                            WHERE RE.ID = :USER_ID
                              AND TO_CHAR(AG.DTAPROPRIACAO, 'DD/MM/YYYY') = :CURRENT_DATE
                            UNION
                           SELECT CO.NRSEQCONTRATO AS ID, 'CONTRATO - ' || CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS NAME,
                                  $appropriationTime AS TOTAL_FAT, 0 AS TOTAL_NOT_FAT,  'B' AS DIFF
                             FROM FCT_ITEMCONTRATO IC
                             JOIN FCT_CONTRATO    CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
                           WHERE 1 = 1
                           $filter
                               )
                 GROUP BY ID, NAME
SQL;

        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

    }

    public function findUserTotalAppropriationAtDay($userId, $currentDate)
    {
        $query = <<<SQL
            SELECT NVL(APPROPRIATE_TOTAL, 0) AS APPROPRIATE_TOTAL
            FROM (SELECT SUM(AG.QTHORASAPROPRIADAS * 3600) AS APPROPRIATE_TOTAL
                    FROM RESOURCE_T            RE
                    JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
                    JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                    JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
                    JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                    WHERE RE.ID = :USER_ID
                    AND TO_CHAR(AG.DTAPROPRIACAO, 'DD/MM/YYYY') = :CURRENT_DATE
                    AND RA.IDSITUARECGEND = '1')
SQL;

        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params)[0];
    }

    public function findMnemonicsValuesRTRByCda($nrseqcontrato, $date, $userId)
	{
		$query = <<<SQL
             SELECT PROPOSTA, SERVICE_NAME, CUSTOMER_REPRESENTATIVE, BILLED_DESLOC, PROJECT_ID, PROJECT_NAME,
                    DATE_CDA, TOTAL, CUSTOMER_NAME, CUSTOMER_ID, USER_NAME,
                    (SELECT HORADEC_HORACHAR(BILLED) FROM DUAL) AS TOTAL_BILLED,
                    (SELECT HORADEC_HORACHAR(UNBILLED) FROM DUAL) AS UNBILLED_TOTAL,
                    (SELECT HORADEC_HORACHAR(BILLED + UNBILLED) FROM DUAL) AS TOTAL_WORK
               FROM (SELECT OM.NRORCAMENTO AS PROPOSTA, NULL AS SERVICE_NAME, OM.NRSEQINTERLOCUTOR AS CUSTOMER_REPRESENTATIVE,
                            NULL AS BILLED_DESLOC, NULL AS PROJECT_ID, NULL AS PROJECT_NAME,
                            TO_CHAR(AG.DTAPROPRIACAO, 'DD/MM/YYYY') AS DATE_CDA, SUM(AG.QTHORASAPROPRIADAS) + 0 AS TOTAL,
                            CL.NMFANTCLIE AS CUSTOMER_NAME, CL.CDCLIENTE AS CUSTOMER_ID,  RE.NAME AS USER_NAME,
                            SUM(AG.QTHORASAPROPRIADAS) AS BILLED, 0 AS UNBILLED
					   FROM RESOURCE_T            RE
					   JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
					   JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
					   JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
					   JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT AND TO_CHAR(AG.DTAPROPRIACAO, 'DD/MM/YYYY') = :DATECDA

					   JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
					   JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
					   JOIN CLIENTE               CL ON CL.NRPARCNEGOCIO = CO.NRPARCNEGOCIO
                  LEFT JOIN FCT_REFEORCACONT      OC ON OC.NRSEQCONTRATO = CO.NRSEQCONTRATO
                  LEFT JOIN FCT_ORCAMENTO         OM ON OM.NRSEQORCAMENTO = OC.NRSEQORCAMENTO

					  WHERE RE.ID = :USER_ID
                        AND CO.NRSEQCONTRATO = :NRSEQCONTRATO
                     GROUP BY TO_CHAR(AG.DTAPROPRIACAO, 'DD/MM/YYYY'), CL.NMFANTCLIE, CL.CDCLIENTE, RE.NAME, OM.NRORCAMENTO, OM.NRSEQINTERLOCUTOR)
SQL;

		$params = array(
			':USER_ID' => $userId,
			':DATECDA' => $date,
			':NRSEQCONTRATO' => $nrseqcontrato
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}


}